;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;;					Mind wandering model (based on van Vugt et al.)
;;
;;					Modified for First-Year Research Project
;;
;;							Daniël van der Hall
;;					 				& 
;;							Maarten van der Velde
;;
;;	----------------------------------------------------------------------------------
;;
;;	Running instructions:
;;	1. Run the file setOutput.py. This sets where the output trace of the model goes.
;; 	2. Open ACT-R and load this model file.
;;	3. 
;;	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;





;; mind wandering task (SART)
;; in this task, participants view a sequence of letters, and they have to press every time, except when it is a rare non-target; in addition, they can get thought probes
;; question 1: how focused they were on the task: 0: on-task, 1: task-related thought, 2: distraction, 3: mind wandering 
;; question 2: Did you know that you were mind wandering or did you notice it when asked? 0=aware, 1=unaware
;; question 3: content of thought: Else: nothing of the following; Emo: emotional thought; Imag: visual mental imagery; Lang: inner speech; Sens: bodily sensation; Son: auditory mental imagery (songs, memory of people's speech)
;; question 4: temporal orientation: Fut: future oriented thought; NoTime: thought in no particular time orientation; Pass: past oriented thought; Pres: present oriented thought
;; 811 digits that are go-trials, 77 3s that are no-go trials
;; stimuli were presented for 500 ms with ISI of 2000 ms
;; 24 thought probes are randomly interspersed between trials
;; Task from Bastian & Sackur (2013)
;; This version also adds thought probe with corresponding responses. The simplest version is to respond "distracted" when in thought pump and "on-task" when not in thought pump. Of course RTs to thought probes are going to be much too fast.
;; This version changes the "attend" chunk that ends the thought pump before flushing it to ensure it doesn't get strengthened
;; To ensure that the model doesn't get stuck, I start the activation of remember-to-attend higher
;; Version 3: This version relative to version 2 changes the recalled thought pump item before the next one is recalled.
;; Version 4: This version also responds to questions about awareness and content of thoughts.

(defvar *responses* nil)
(defvar *done*  nil)
(defvar *response-times* nil)
(defvar *trialind* nil)
(defvar *allresp* nil)
(defvar *allrt* nil)
(defvar *stimuli* nil)
(defvar *start-time* nil)
(defvar *rts* nil)
(defvar *curtrial* nil)

(defun run-exp ()
(with-open-file (*standard-output* #p"/Users/Maarten/Dropbox/First-Year Research Project/ACT-R/output_tests/depressed_p10_beh.txt" :direction :output) (do-exp))

)

(defun do-exp ()
;; runs a sequence of trials.
	(reset)
	(setf *allresp* nil)
	(setf *allrt* nil)
	(setf *trialind* nil)
	(setf *curtrial* 0)
	(setf *stimuli* (permute-list (concatenate 'list (make-array 405 :initial-element "d") (make-array 37 :initial-element "3") (make-array 12 :initial-element "Q"))))
	; Small stimulus set for testing: 2 digits, 2 threes, 2 thought probes.
	;(setf *stimuli* (permute-list (concatenate 'list (make-array 2 :initial-element "d") (make-array 2 :initial-element "3") (make-array 2 :initial-element "Q"))))
	(loop for str in *stimuli* do (do-trial str))

	; print the behavioral results
	(format t "mean ACC: ~a~%"  (perc *allresp* (reverse *stimuli*)))
	(print *allrt*)
	(print (reverse *stimuli*))
	(print *allresp*)
)

(defun do-trial (str)
;; runs one trial. str can be either "3" or "d" (which is any other digit) or "Q", which is the thought probe.
	(let 
		((window (open-exp-window "SART Experiment"									  
								 :visible nil
								 :width 300
								 :height 300
								 :x 300
								 :y 300)))

		(if (equal str "Q")
		(add-text-to-exp-window :text "what are you thinking about?"
								:width 300
								:height 30
								:x 145
								:y 150)
		(add-text-to-exp-window :text str
								:width 30
								:height 30
								:x 145
								:y 150)
	)

	(print str)
	(install-device window)
	(setf *responses* nil)
	(incf *curtrial*)
	(setf *response-times* nil)
	(setf *start-time* (get-time))	
	(when *actr-enabled-p*
		(proc-display)
		(run 2)) ;; each trial is 2 seconds; stimulus presented for 500 ms
	(clear-exp-window)
	(proc-display))

	; if response time is nil we need to add the current trial to the trial index, since it normally only happens with pressing a key
	(if (null *response-times*) (push *curtrial* *trialind*))
  
	(print *response-times*)
	(push *responses* *allresp*)
	(push *response-times* *allrt*)
)

(defmethod rpm-window-key-event-handler ((win rpm-window) key)
  (unless (string= key " ")
	(push (string key) *responses*)
	(push *curtrial* *trialind*)
	(push (- (get-time) *start-time*) *response-times*)
	))

;;;; analysis tools ;;;;
(defun get-expected-responses (stims)
		(loop for elem-index from 0 below (length stims) do 
			  
			  (let ((current-elem (nth elem-index stims)))
					 (cond
						 ((string= current-elem "d")
							 (setf (nth elem-index stims) "f")
						 )
						 ((string= current-elem "3")
							 (setf (nth elem-index stims) nil)
						 )
						 ((string= current-elem "Q")
							 (setf (nth elem-index stims) "Q")
						 )
					 )
				)
		)
		stims
)

(defun perc (resp stim)
;; check for how many instances the response nil is given to 3 and the response "f" is given to any other digit "d"
  (let ((s 0.0) 
		(correctresp (get-expected-responses stim)))
	(dolist (i resp)
	  (when (equalp i (pop correctresp)) (incf s 1)))
	(/ s (length resp))))

(defun num-incorrect-omissions (resp stim)
;; add up the number of omission trials
  (let ((s 0.0) 
		(correctresp (get-expected-responses stim)))
	(print correctresp)
	(print resp)
	(dolist (i resp)
	  (when (and (equalp i "3") (not (equalp i (pop correctresp)))) (incf s 1)))
	s)
)

(defun avg (numbers)
  (setf numbersnotnil (remove nil numbers))
  (if (null numbersnotnil)
	  (error "Average of the empty list is undefined.")
	  (float (/ (reduce #'+ numbersnotnil)
		 (length numbersnotnil)))))


(defmacro square (x)
  `(* ,x ,x))

(defun var (numbers)
  (setf numbersnotnil (remove nil numbers))
  ;(print numbers)
  (let ((mean (avg numbersnotnil))
		(n (length numbersnotnil)))
	(/ (reduce #'+ (map 'list #'(lambda (x) (square (- mean x))) numbersnotnil))
	   (1- n))))


(clear-all)

(define-model mindwandering

(sgp :v "/Users/Maarten/Dropbox/First-Year Research Project/ACT-R/output_tests/depressed_p10_trace.txt" ;t ; tracing verbose 
	 :rt -3 ; retrieval threshold
	 :trace-detail low 
	 :save-buffer-trace t
	 :sact t ; save activation values
	 :imaginal-action-activation 1.0 ; make sure that imaginal-action also causes spreading activation
	 ;  :visual-location-activation 1.0 ;this makes sure that any appearance of info in the visual field can nudge the participant back to attending
	 ; make sure that a person does not respond to the stimulus too often by requiring them to only respond to a stimulus when it appears recently 
	 :visual-onset-span 0.7  ; set to 0.7 on 13/11
	 :show-focus t
	 :act low ;t/medium/low/nil ;;tracing activation values
	 :er t ;;;if er=t then ties will be broken randomly; if er=nil then always the same production will be chosen with ties
	 :mas 2 ;; mas: activate spreading activation
	 :esc t
	 :declarative-num-finsts 4 ;; :declarative-num-finsts: number of items that are kept as recently retrieved (default 4)
	 :declarative-finst-span 3 ;; :declarative-finst-span: how long items stay in the recently-retrieved state (default 3)
	 :lf 0.85 ; latency factor used to turn activation values into retrieval times
	 ; set from 1 to 0.85 on 11/6/15
	 ; set from .85 to .75 on 16/10/15
	 :ans 0.13 ;activation noise turned from 0.2 to .15 on 28 mar; 0.12 on 15 may
	 :visual-attention-latency 0
	 :ul t ;; sgp :ul t -> now the program can learn utilities when certain productions are associated with a reward; In meditation, paying attention for a certain amount of time should lead to a reward. 
	 :egs 0.1 ; :egs expected gain in utility: noise in utility computation
	 :model-warnings nil 
	 :bll 0.5 ;; :bll = set base-level learning, such that items more frequently/recently retrieved gain higher activation
	 ; parameters for base-level inhibition
	 ;:enable-inhibition nil ; turn on the base-level inhibition module
	 ;:inhibition-scale 10 ; 10 is best from Lebiere & Best
	 ;:inhibition-decay 1 ; 1 is best from Lebiere & Best
	 :ol nil  ; set optimized learning to nil to ensure that base-level inhibition works
)

(chunk-type goal state)
(chunk-type lemma value)
(chunk-type srmapping value hand)
(chunk-type item name valence time)
(chunk-type distracted state)
(chunk-type beginning state)
(chunk-type see-thoughts state)
(chunk-type planning state)
(chunk-type subgoal state)

(add-dm
	(lemma isa chunk)
	(left isa chunk)
	(right isa chunk)
	(Olemma isa lemma value "d")
	(Qlemma isa lemma value "3")
	(thoughtProbe isa lemma value "what")
	(Opress isa srmapping value "d" hand left)
	(OntaskPress isa srmapping value "what" hand left)
	(Qnone isa srmapping value "3" hand nil)
	(attending isa chunk)
	(yes isa chunk)
	(no isa chunk)
	
	 (pos isa chunk)
	 (neg isa chunk)
	; (neut isa chunk)
	; Expanded emotional states as per Wigman 2015
	(chrf isa chunk) ; cheerful
	(ins isa chunk)  ; insecure
	(con isa chunk)  ; content
	(dwn isa chunk)  ; down
	(sus isa chunk)   ; suspicious
	(neut isa chunk) ; neutral
	
	(present isa chunk)
	(past isa chunk)
	(future isa chunk)
		
	;; populate the thought pump
	; (itemposA ISA item name "posA" valence pos time present)
	; (itemposS ISA item name "posS" valence pos time present)
	; (item1 ISA item name "1" valence pos time future)
	; (item2 ISA item name "2" valence pos time future)
	; (item3 ISA item name "3" valence pos time future)
	; (item6 ISA item name "6" valence pos time future)
	; (item7 ISA item name "7" valence pos time present)
	; (item8 ISA item name "8" valence pos time past)
	; (item9 ISA item name "9" valence pos time past)
	; (item10 ISA item name "10" valence pos time past)

	; (itemnegV ISA item name "negV" valence neg time present)
	; (itemnegS ISA item name "negS" valence neg time past)
	; (item13 ISA item name "13" valence neg time present)
	; (item14 ISA item name "14" valence neg time present)
	; (item15 ISA item name "15" valence neg time past)
	; (item16 ISA item name "16" valence neg time past)
	; (item17 ISA item name "17" valence neg time past)
	; (item18 ISA item name "18" valence neg time future)
	; (item19 ISA item name "19" valence neg time future)
	; (item22 ISA item name "22" valence neg time future)

	; (itemneuA ISA item name "neuA" valence neut time present)
	; (itemneuV ISA item name "neuV" valence neut time present)
	; (itemneuS ISA item name "neuS" valence neut time present)
	; (item25 ISA item name "25" valence neut time past)
	; (item26 ISA item name "26" valence neut time past)
	; (item27 ISA item name "27" valence neut time past)
	; (item28 ISA item name "28" valence neut time past)
	; (item29 ISA item name "29" valence neut time future)
	; (item30 ISA item name "30" valence neut time future)
	; (item33 ISA item name "33" valence neut time future)
	; (item34 ISA item name "34" valence neut time future)
	
	; Expanded emotional states as per Wigman 2015 - balances were copied from the 'pos' and  'neg' definitions in the original version.
	; dynamic dm definitions (success)
	; chrf
	(item1chrf ISA item name "chrf1" valence chrf time past)
	 (item2chrf ISA item name "chrf2" valence chrf time past)
	 (item3chrf ISA item name "chrf3" valence chrf time past)
	 (item4chrf ISA item name "chrf4" valence chrf time past)
	 (item5chrf ISA item name "chrf5" valence chrf time past)
;	 (item6chrf ISA item name "chrf6" valence chrf time past)
;	 (item7chrf ISA item name "chrf7" valence chrf time past)
;	 (item8chrf ISA item name "chrf8" valence chrf time past)
;	 (item9chrf ISA item name "chrf9" valence chrf time past)
;	 (item10chrf ISA item name "chrf10" valence chrf time past)

	; ins
	(item1ins ISA item name "ins1" valence ins time past)
	 (item2ins ISA item name "ins2" valence ins time past)
	 (item3ins ISA item name "ins3" valence ins time past)
	 (item4ins ISA item name "ins4" valence ins time past)
	 (item5ins ISA item name "ins5" valence ins time past)
	 (item6ins ISA item name "ins6" valence ins time past)
	 (item7ins ISA item name "ins7" valence ins time past)
	 (item8ins ISA item name "ins8" valence ins time past)
	 (item9ins ISA item name "ins9" valence ins time past)
	 (item10ins ISA item name "ins10" valence ins time past)
	 (item11ins ISA item name "ins11" valence ins time past)
	 (item12ins ISA item name "ins12" valence ins time past)
	 (item13ins ISA item name "ins13" valence ins time past)
	 (item14ins ISA item name "ins14" valence ins time past)
	 (item15ins ISA item name "ins15" valence ins time past)

	; con
	(item1con ISA item name "con1" valence con time past)
	 (item2con ISA item name "con2" valence con time past)
	 (item3con ISA item name "con3" valence con time past)
	 (item4con ISA item name "con4" valence con time past)
	 (item5con ISA item name "con5" valence con time past)
;	 (item6con ISA item name "con6" valence con time past)
;	 (item7con ISA item name "con7" valence con time past)
;	 (item8con ISA item name "con8" valence con time past)
;	 (item9con ISA item name "con9" valence con time past)
;	 (item10con ISA item name "con10" valence con time past)

	; dwn
	(item1dwn ISA item name "dwn1" valence dwn time past)
	 (item2dwn ISA item name "dwn2" valence dwn time past)
	 (item3dwn ISA item name "dwn3" valence dwn time past)
	 (item4dwn ISA item name "dwn4" valence dwn time past)
	 (item5dwn ISA item name "dwn5" valence dwn time past)
	 (item6dwn ISA item name "dwn6" valence dwn time past)
	 (item7dwn ISA item name "dwn7" valence dwn time past)
	 (item8dwn ISA item name "dwn8" valence dwn time past)
	 (item9dwn ISA item name "dwn9" valence dwn time past)
	 (item10dwn ISA item name "dwn10" valence dwn time past)
	 (item11dwn ISA item name "dwn11" valence dwn time past)
	 (item12dwn ISA item name "dwn12" valence dwn time past)
	 (item13dwn ISA item name "dwn13" valence dwn time past)
	 (item14dwn ISA item name "dwn14" valence dwn time past)
	 (item15dwn ISA item name "dwn15" valence dwn time past)

	; sus
	(item1sus ISA item name "sus1" valence sus time past)
	 (item2sus ISA item name "sus2" valence sus time past)
	 (item3sus ISA item name "sus3" valence sus time past)
	 (item4sus ISA item name "sus4" valence sus time past)
	 (item5sus ISA item name "sus5" valence sus time past)
	 (item6sus ISA item name "sus6" valence sus time past)
	 (item7sus ISA item name "sus7" valence sus time past)
	 (item8sus ISA item name "sus8" valence sus time past)
	 (item9sus ISA item name "sus9" valence sus time past)
	 (item10sus ISA item name "sus10" valence sus time past)
	 (item11sus ISA item name "sus11" valence sus time past)
	 (item12sus ISA item name "sus12" valence sus time past)
	 (item13sus ISA item name "sus13" valence sus time past)
	 (item14sus ISA item name "sus14" valence sus time past)
	 (item15sus ISA item name "sus15" valence sus time past)


	;(itemAttend ISA item name "attend" valence pos time present)
	
	; Expanded emotional states as per Wigman 2015
	; altered to equally positive term? might also be cheerful.. -D
	(itemAttendPos ISA item name "attend" valence pos time present)
	(itemAttendNeg ISA item name "attend" valence neg time present)
	
	(attend-object isa goal state attend-to-object)
	(distracted isa goal state get-distracted)
	(startgoal isa beginning state start)
	(incgoal isa planning state strengthen-goal)
) 


(set-base-levels 
	(attend-object 600 -10000)
	(distracted 200 -10000)
	(olemma 100000 -1000000)
	(qlemma 100000 -1000000)
	(thoughtprobe 10000 -1000000)
	(opress 100000 -1000000)
	(onTaskPress 10000 -1000000)
	(qnone  100000 -1000000)
 
	; expanded according to the DM definitions above
	; dynamic base-levels (success)
	; chrf
	(item1chrf 200.0 -100000.0)
	 (item2chrf 200.0 -100000.0)
	 (item3chrf 200.0 -100000.0)
	 (item4chrf 200.0 -100000.0)
	 (item5chrf 200.0 -100000.0)
;	 (item6chrf 200.0 -100000.0)
;	 (item7chrf 200.0 -100000.0)
;	 (item8chrf 200.0 -100000.0)
;	 (item9chrf 200.0 -100000.0)
;	 (item10chrf 200.0 -100000.0)

	; ins
	(item1ins 200.0 -100000.0)
	 (item2ins 200.0 -100000.0)
	 (item3ins 200.0 -100000.0)
	 (item4ins 200.0 -100000.0)
	 (item5ins 200.0 -100000.0)
	 (item6ins 200.0 -100000.0)
	 (item7ins 200.0 -100000.0)
	 (item8ins 200.0 -100000.0)
	 (item9ins 200.0 -100000.0)
	 (item10ins 200.0 -100000.0)
	 (item11ins 200.0 -100000.0)
	 (item12ins 200.0 -100000.0)
	 (item13ins 200.0 -100000.0)
	 (item14ins 200.0 -100000.0)
	 (item15ins 200.0 -100000.0)
	 
	; con
	(item1con 200.0 -100000.0)
	 (item2con 200.0 -100000.0)
	 (item3con 200.0 -100000.0)
	 (item4con 200.0 -100000.0)
	 (item5con 200.0 -100000.0)
;	 (item6con 200.0 -100000.0)
;	 (item7con 200.0 -100000.0)
;	 (item8con 200.0 -100000.0)
;	 (item9con 200.0 -100000.0)
;	 (item10con 200.0 -100000.0)

	; dwn
	(item1dwn 200.0 -100000.0)
	 (item2dwn 200.0 -100000.0)
	 (item3dwn 200.0 -100000.0)
	 (item4dwn 200.0 -100000.0)
	 (item5dwn 200.0 -100000.0)
	 (item6dwn 200.0 -100000.0)
	 (item7dwn 200.0 -100000.0)
	 (item8dwn 200.0 -100000.0)
	 (item9dwn 200.0 -100000.0)
	 (item10dwn 200.0 -100000.0)
	 (item11dwn 200.0 -100000.0)
	 (item12dwn 200.0 -100000.0)
	 (item13dwn 200.0 -100000.0)
	 (item14dwn 200.0 -100000.0)
	 (item15dwn 200.0 -100000.0)

	; sus
	(item1sus 200.0 -100000.0)
	 (item2sus 200.0 -100000.0)
	 (item3sus 200.0 -100000.0)
	 (item4sus 200.0 -100000.0)
	 (item5sus 200.0 -100000.0)
	 (item6sus 200.0 -100000.0)
	 (item7sus 200.0 -100000.0)
	 (item8sus 200.0 -100000.0)
	 (item9sus 200.0 -100000.0)
	 (item10sus 200.0 -100000.0)
	 (item11sus 200.0 -100000.0)
	 (item12sus 200.0 -100000.0)
	 (item13sus 200.0 -100000.0)
	 (item14sus 200.0 -100000.0)
	 (item15sus 200.0 -100000.0)
	
	(itemAttendPos 220 -100000)
	(itemAttendNeg 220 -100000)
)

;; start the task by retrieving the attend goal and strengthening it
(P intend-to-attend-goal
	=goal>
		isa			beginning
		state		start
	?retrieval>
		buffer		empty
		state		free
		- state		error
==>
	+retrieval>
		isa 		goal
		state 		attend-to-object
	-goal>
)

;; retrieve the current goal (done many times while paying attention, until the visual location buffer is stuffed)
;; the funny retrieval syntax in the action part of the production is intended to prevent that the goal is not strengthened during the retrieval
(p check-current-goal
	?retrieval>
		state   	free
		- state   	error
	=retrieval>
		isa 		goal
		state 		=statename
		- state 	get-distracted
	?goal>
		buffer 		empty
	?visual>
		- scene-change T
==>   
	=retrieval>
		state 		nil
	-retrieval>
	+retrieval>
		isa 		goal
		- state 	nil
)

;; attend to a stimulus when there has just been a scene change
(P attend-stim
	=retrieval>
		isa			goal
		state		attend-to-object
	?retrieval>
		state		free
		- state	 	error
	?visual>
		state		free
		scene-change T
	?goal>
		buffer 		empty
==>
	+goal>
		isa			subgoal
		state		identify
	+visual-location>
		isa		 	visual-location
		screen-x	lowest
		screen-y	lowest
	; this part ensures that the attend-object goal does not get strengthened when it gets flushed
	=retrieval> 
		state 		nil
	-retrieval>
)

(P identify-stim
	=goal>
		isa			subgoal
		state		identify
	=visual-location>
		isa			visual-location
	?visual>
		state		free
==>
	+visual>
		isa		 	move-attention
		screen-pos	=visual-location
	+goal>
		isa			subgoal
		state		check-S-R
)

;; check-whether-O
(P check-whether-O
	=visual>
		isa			text
		value		=val
	=goal>
		isa			subgoal
		state		check-S-R
	?retrieval>
		state		free
		- state		error
==>
	+retrieval>
		isa			lemma
		value		=val
	+goal>
		isa			subgoal
		state		get-S-R-result
	+visual>
		isa			clear-scene-change
)

(P get-result-of-check
	=retrieval>
		isa			lemma
		value		=val
	=goal>
		isa			subgoal
		state		get-S-R-result
==>
	+retrieval>
		isa			srmapping
		value		=val
	+goal>
		isa			subgoal
		state		do-response
)

; implement response if "d" ; then retrieve a new goal
(P respond
	?manual>
		state		free
	=retrieval>
		isa			srmapping
		value		=val
		- value		"what" 
		hand		=hand
	=goal>
		isa			subgoal
		state		do-response
==>
	+manual>
		isa			press-key
		key			"f"
		!output!	"f"
	-goal>
	-visual-location>
	-visual>
	+retrieval>
		isa			goal
		- state		nil
)

;; series of productions when responding to a thought probe
; implement response to a thought probe when paying attention: "on-task"
(P respond-on-task
	?manual>
		state		free
	=retrieval>
		isa			srmapping
		value		"what"
		hand		=hand
	=goal>
		isa			subgoal
		state		do-response
==>
	+manual>
		isa			press-key
		key			"1"
		!output!	"1"
	+goal>
		isa			subgoal
		state		respond-awareness
)

(P respond-on-awareness
	?manual>
		state		free
	=goal>
		isa			subgoal
		state		respond-awareness
==>
	+manual>
		isa			press-key
		key			"a"
		!output!	"a"
	+goal>
		isa			subgoal
		state		respond-time	  
)

; after finishing all the responses, go back to the on-task state
(P respond-on-time
	?manual>
		state		free   
	=goal>
		isa			subgoal
		state		respond-time
==>
	+manual>
		isa			press-key
		key			"p"
		!output!	"p"
	+goal>
		isa			beginning
		state		start
	-retrieval>
	-visual-location>
	-visual>
)

; implement no-response if "3"
(P do-not-respond
	=retrieval>
		isa			srmapping
		value		=val
		hand		nil
	=goal>
		isa			subgoal
		state		do-response
==>
	-goal>
	-visual-location>
	-visual>
	+retrieval>
		isa			goal
		- state		nil
)



;; PRODUCTIONS FOR THE THOUGHT PUMP THAT REFLECTS THE DISTRACTION
(p start-thought-pump
	=retrieval>
		isa			goal
		state		get-distracted
==>
	+goal>
		isa			distracted
		state		remember-items
	+retrieval>
		isa			item
		- time		nil
		:recently-retrieved nil
	-imaginal>
)


;; continue thought pump by retrieving subsequent memories
;; follow-up memories occur when the time is past or future; when time is present, then return to meditation object
(p recall-next
	=goal>
		isa			distracted
		state		remember-items
	=retrieval>
		isa			item
		name		=itemname
		- name		"attend"
		valence		=val
		time		=time
	?retrieval>
		state		free
		- state 	error
==>
	; change the current memory so that it does not increase in activation
	=retrieval>
		time		nil
	; move the current memory in the imaginal-action, from where it can spread activation
	=imaginal-action> =retrieval
	; retrieve the next item, which is different from the one recently recalled
	+retrieval>
		isa			item
		- name		=itemname
		- time		nil
		:recently-retrieved nil
	; only ask for items that were not recently retrieved to prevent one item from dominating the model  
	=goal>
)

; respond the default when you have not retrieved SR mapping and when you are distracted and something appears in the visual-location buffer
; also make sure that you are not busy retrieving when responding with default
(P respond-default
	?manual>
		state		free
	=visual-location>
		isa			visual-location
		< width		10
	=goal>
		isa			distracted
		state		react-to-stim
	?visual>
		state 		free
	?retrieval>
		state 		free
		- state 	error
==>
	+manual>
		isa			press-key
		key			"f"
		!output!	"f"
	+goal>
		isa			distracted
		state		remember-items
	+visual>
		isa			move-attention
		screen-pos	=visual-location
	-visual-location>
	-visual>
)

(p notice-stim-while-distracted
	=goal>
		isa			distracted
		state 		remember-items
	?visual>
		state		free
		scene-change T
	?retrieval>
		state		free
		-  state 	error
	?visual-location>
		- buffer 	empty
==>
	+visual-location>
		isa			visual-location 
		screen-x	lowest
		screen-y	lowest
	+goal>
		isa			distracted
		state		react-to-stim
)

;; when there is a large stimulus, this awakes the person from their distraction. These are the thought probes
(p see-probe-distracted
	=goal>
		isa			distracted
		state		react-to-stim
	=visual-location>
		isa			visual-location
		> width		10
	?visual>
		state		free
	?retrieval>
		state		free
		- state		error
==>
	+visual>
		isa			move-attention
		screen-pos	=visual-location
	+goal>
		isa			subgoal
		state 		prep-to-respond
 )


;; respond distracted, then inspect the nature of thoughts
(p respond-to-probe
	=goal>
		isa			subgoal
		state		prep-to-respond
	?manual>
		state		free
	=retrieval>
		isa			item
	?retrieval>
		state		free
		- state		error   
==>
	=retrieval>
	+manual>
		isa			press-key
		key			"o"
		!output!	"o"
	+goal>
		isa			subgoal
		state		respond-aware   
)

(p respond-aware-offtask
	=goal>
		isa			subgoal
		state		respond-aware
	?manual>
		state		free
	=retrieval>
		isa			item
	?retrieval>
		state		free
		- state		error
==>
	=retrieval>   
	+manual>
		isa			press-key
		key			"u"
		!output!	"u"
	+goal>
		isa			subgoal
		state		respond-time-from-memory   
)


; after we have responded to the thought probe, we assume the person finishes mind-wandering and is back on-task again
; the response on this question depends on the thought that is currently in the retrieval buffer
(p respond-time-past
	=goal>
		isa			subgoal
		state		respond-time-from-memory
	=retrieval>
		isa			item
		name 		=itemname
		valence		=val
		time 		past
	?retrieval>
		state 		free
		- state 	error
	?manual>
		state		free  
==>
	; then respond and get ready to retrieve the attending goal
	=retrieval>
		name		"done"
	+manual>
		isa			press-key
		key			"a"
		!output!	"a"  
	-imaginal-action>
	+goal>
		isa			beginning
		state		start
	-retrieval>
)

(p respond-time-present
	=goal>
		isa 		subgoal
		state 		respond-time-from-memory
	=retrieval>
		isa 		item
		name 		=itemname
		valence 	=val
		time 		present
	?retrieval>
		state 		free
		- state 	error
	?manual>
		state  		free
==>
	; then respond and get ready to retrieve the attending goal
	=retrieval>
		name		"done"
	+manual>
		isa			press-key
		key			"p"
		!output!	"p"  
	-imaginal-action>
	+goal>
		isa 		beginning
		state 		start
	-retrieval>   
)

(p respond-time-future
	=goal>
		isa 		subgoal
		state 		respond-time-from-memory
	=retrieval>
		isa 		item
		name 		=itemname
		valence 	=val
		time 		future
	?retrieval>
		state 		free
		- state 	error
	?manual>
		state  		free
==>
	; then respond and get ready to retrieve the attending goal
	=retrieval>
		name		"done"
	+manual>
		isa			press-key
		key			"f"
		!output!	"f"  
	-imaginal-action>
	+goal>
		isa 		beginning
		state 		start
	-retrieval>  
)

;; if the remembered item is the attention object, return to pay-attention (with some probability), which brings you back to the task
(p remember-to-attend
	=goal>
		isa 		distracted
		state 		remember-items
	?retrieval>
		state 		free
	=retrieval>
		isa 		item
		name 		"attend"
		valence 	=val
		time 		=time
==>
	; then get ready to retrieve the attending goal
	=retrieval>
		name 		"done"
	-imaginal-action>
	+goal>
		isa 		beginning
		state 		start
	-retrieval>
)

(goal-focus startgoal)

(setf *actr-enabled-p* t)

; add connection weights, which make it more likely to remember a positive item once a positive item has been retrieved, and vice versa.
(add-sji
	; Adapted for 5 mental states as per Wigman 2015
	; (distracted itemnegV 2)
	; (distracted itemneuV 2)
	;;;(distracted item1ins 2)
	;;;(distracted item1dwn 2)
	;;;(distracted item1sus 2)
	
	(lemma itemattendPos 0.1) ;; changed from 0.5
	(lemma itemattendNeg 0.1) ;; changed from 0.5
	(lemma attend-object 0.1))

; to encourage the thought pump to end, we make it so that some retrieved memories prime the "attending" chunk
(add-sji
	; Adapted for 5 mental states as per Wigman 2015
	; (pos itemattend .2) 
	; (neg itemattend .1)
	; (neut itemattend .1)
	
	; dynamic sji (success)
	;;;(chrf itemattend 0.1)
	;;;(ins itemattend 0.1)
	;;;(con itemattend 0.1)
	;;;(dwn itemattend 0.1)
	;;;(sus itemattend 0.1)
	;(chrf chrf 1)
	(chrf con 0.1)
	(chrf dwn 0.01)
	(chrf ins 0.01)
	(chrf sus 0.01)
	(chrf itemattendPos 0.01)
	
	(con chrf 0.1)
	;(con con 0.2)
	(con dwn 0.01)
	(con ins 0.01)
	(con sus 0.01)
	(con itemattendPos 0.01)
	
	(dwn chrf 0.01)
	(dwn con 0.01)
	;(dwn dwn 0.2)
	(dwn ins 0.1)
	(dwn sus 0.1)
	(dwn itemattendNeg 0.01)
	
	(ins chrf 0.01)
	(ins con 0.01)
	(ins dwn 0.1)
	;(ins ins 0.2)
	(ins sus 0.1)
	(ins itemattendNeg 0.01)
	
	(sus chrf 0.01)
	(sus con 0.01)
	(sus dwn 0.1)
	(sus ins 0.1)
	;(sus sus 0.2)
	(sus itemattendNeg 0.01)
	
	;(chrf itemattendPos 0.001)
	;(con itemattendPos 0.001)
	;(dwn itemAttendNeg 0.001)
	;(ins itemAttendNeg 0.001)
	;(sus itemAttendNeg 0.001)

	
	;;;(present itemattend .2)
	;;;(past itemattend .1)
	;;;(future itemattend .1)
	)

; set utility of notice-stim-while-distracted such that it is likely to fire (instead of recall-next)
(spp notice-stim-while-distracted :u 0.8))